/* 複素ニュートン法 */
#include <stdio.h>
#include <math.h>

typedef struct {
  double re;
  double im;
} Complex;

/* 複素ニュートン法を実行する関数 */
Complex newton(Complex, double);

/* 本プログラムで使用する複素数の演算のみ記述 */
Complex csub(Complex, Complex);
Complex cmul(Complex, Complex);
Complex cdiv(Complex, Complex);

/* f(x) */
Complex f(Complex);

/* f(x)の導関数 */
Complex df(Complex);

int main()
{
  Complex x={-1.0,1.0};
  Complex y;
  double eps=1.0e-6;
  y=newton(x,eps);
  printf("(%.10f, %.10f)\n",y.re,y.im);
}

/* 複素ニュートン法を実行する関数 */
Complex newton(Complex x, double eps)
{
  double err;
  Complex y;
  err=1.0;
  while(err > eps){
    y=csub(x,cdiv(f(x),df(x)));
    err=sqrt((x.re-y.re)*(x.re-y.re)+
	(x.im-y.im)*(x.im-y.im));
    x=y;
  }
  return(x);
}

/* f(x)=x^3 - 1 */
Complex f( Complex x )
{
  Complex y;
  Complex u={1.0,0.0};
  y=cmul(x,x); /* y=x*x */
  y=cmul(y,x); /* y=y*x=x*x*x */
  y=csub(y,u); /* y=y-1=x*x*x-1 */
  
  return(y);
}

/* f'(x)=3x^2 */
Complex df( Complex x )
{
  Complex y;
  Complex u={3.0,0.0};

  y=cmul(x,x);
  y=cmul(y,u);
  return(y);
}
Complex csub(Complex z, Complex w)
{
  Complex t;
  t.re=z.re-w.re;
  t.im=z.im-w.im;

  return(t);
}

Complex cmul(Complex z, Complex w)
{
  Complex t;
  t.re=z.re*w.re-z.im*w.im;
  t.im=z.re*w.im+z.im*w.re;

  return(t);
}

Complex cdiv(Complex z, Complex w)
{
  Complex t;
  double s;
  s=w.re*w.re+w.im*w.im;

  t.re=(z.re*w.re+z.im*w.im)/s;
  t.im=(z.im*w.re-z.re*w.im)/s;

  return(t);
}
